#include <system.h>
#include "50.h"

/*FINAL CONTROL ROUTINE, "PowerCop" Cruiser Power Regulation System
This software performs the following tasks:
	*Checks current being drawn off of auxilliary electrical devices at regular intervals using
	A/D conversion and the known I/V characteristics of our sensors.
	*Checks alternator current, battery current, and battery voltage to estimate whether the batttery is being
	charged or discharged, as well as providing a rough estimate of the state of charge.
	*Shuts off loads depending on the results of these measurements to prevent battery from being overdrawn
	*Writes the current being drawn from the loads, and any potential warnings about battery status and the
	time of each measurement, to an external EEPROM memory chip.
	*Provides a user interface to a series of buttons and an LCD screen, which allows users to set the system
	time, establish thresholds for the various device lines, and check the current being drawn at the moment by a
	given line.
	*Allows the user to connect the system to any PC with a serial port and HyperTerminal and upload the results
	stored in memory.  These results can be easily moved into an Excel spreadsheet.*/

#pragma DATA _CONFIG1H, _OSC_HS_1H //10 mhz clock on ucontroller
#pragma DATA _CONFIG2H, _WDTPS_16384_2H //approx. 1 min delay before wakes on WDT
#pragma DATA _CONFIG4L, _LVP_OFF_4L 	
#pragma DATA _CONFIG3H, _MCLRE_ON_3H

volatile bit user_int@INTCON3.0;	//B1 interrupt to enter user interface
bool user_flag = false;

volatile bit time_out@INTCON.2;		//timer zero interrupt to time-out user menu
bool lcd_menu_flag = false;

short timeoutflag = 0; 				//semaphores for handling interrupts
short twentyfour = 0;
unsigned short over = 0;			//global system override variable
unsigned short term_read = 0;		//global variable specifying terminal accessed

void interrupt(void)
{
	if (user_int)
	{
		user_int = 0;				//handling user interrupt 
		user_flag = true;
	}
	
    
    if (time_out)
    {
		time_out = 0;				//handling timer zero timeout
		timeoutflag = 1;
		//LCD_setpos(1,0);
		//LCD_printf("Timer0 interrupt");
    
    }         
	
}

void main(void)
{
	unsigned short minutes = 0x09;		//initialize time clock when system turns on
	unsigned short hours = 0x13;
	unsigned short date = 0x03;
	unsigned short month = 0x05;
	
										//initialization commands

	int_wdt_init();	//interrupts, WDT						
	LCD_init();		//LCD
	term_init();	//serial port
	ad_init();		//A/D conversion
	i2c_init();		//I2C (for time chip, memory chip)
	io_init();		//determines which pins will switch on and off
	time_init_clk(minutes, hours, date, month);	//set time
	time_init_set();	//time chip settings
	
	//time_out_init();
	menu_init();	//user menu functions
	/*disable timer o for time being*/
	intcon.5 = 0;	//disable timer 0 at beginning of main
	
	unsigned short feedback = 0;	//deprecated; not used
	bool car_on = true;		//This will trigger an "off" state in control when the unit begins
	unsigned short over = 0;		//System defaults as override
	bool control = false;	//signals if main program has decided to control things
	unsigned short fault = 0;	//warning about potential system failure
	unsigned short soc = 90;	//initial setting for SOC (will be changed quickly
	bool read_all = false;		//tells terminal function if it needs tor read all entries
	
	unsigned short alt_current = 0;	//used to check if car is on.
	
	unsigned short mode;		//deprecated; no longer used
	unsigned short switch_status;	//deprecated; no longer used
	unsigned short wait_time = 1;	//initially, wait time is 1 minute to get into control quickly
	unsigned short wait_time_1 = 30;	//wait time if car if on for a while
	unsigned short wait_time_2 = 15;	//wait time if car has just been turned on/off
	unsigned short wait_time_3 = 60;	//wait time if car has been off for a while
	unsigned short time_track = 0;		//Allows system to determine mode of operation on its own shortly after startup
	unsigned short inc = 0;				//deprecated; not used
	unsigned short dummy = 0;			//used to satisfy i2c_time when not in
										//memory write mode
	
	unsigned short size = 22;			//spacing between first entries of con
										//secutive data writes.
	
	unsigned short address_low_start = 0x00;	//memory pointers
	unsigned short address_high_start = 0x00;
	unsigned short address_low_inc = 0x00;
	unsigned short address_high_inc = 0x00;
	
	//AD_pin_assignments.  May need to be moved...
	unsigned short L1_ad = 0x01;	//Camera--approx. 7.5 A (Only for monitoring purposes--1)
	unsigned short L2_ad = 0x05;	//GPS Transciever--15 A (best guess--could not tear up car enough to know for sure--3)
	unsigned short L3_ad = 0x09;	//Motorola 2-way radio--15 A (2)
	unsigned short R1_ad = 0x0D;	//Interceptor Main Power / siren (4)
	unsigned short R2_ad = 0x11;	//floods on lightbar, computer, printer (4)
	unsigned short R3_ad = 0x15;	//Power ports (4)
	unsigned short alt_ad = 0x19;	//alternator current
	unsigned short bata_ad = 0x1D;	//battery current
	unsigned short batv_ad = 0x21;	//battery voltage
	
	while(1){
		
		/*If current on alternator is greater than some threshold (car on), set wait_time to
		wait_time_1.  If not (car is off), set it to wait_time_2.*/
		
			LCD_setpos(0,0);
			if(fault == 0 && over == false){	//prints system status to screen
				LCD_printf("System Armed");		//active
			}
			if(fault == 0 && over == true){
				LCD_printf("System Idle");		//override
			}
			if(fault == 1){
				LCD_printf("! Battery SOC @ ");	//bad battery
				LCD_int(soc);
				LCD_printf("% !");
			}
			if(fault == 2){
				LCD_printf("! Alternator Fault !");	//bad alternator
			}
			if(fault == 3){
				LCD_printf("ATTN: BATT. FAIL");		//both faults
				}
			LCD_setpos(1,0);
			dummy = i2c_time(false, 0x00, 0x00);	//print time
			int_wdt_init_sleep();					//sleep mode settings
			sleep();								//sleep
			int_wdt_init();							//"active" settings
			if(user_flag == false){					//if timer wakes up
				time_track = time_track + 1;		//increment
				if(time_track >= wait_time){
					if (car_on == false){			//off a while time
						wait_time = wait_time_3;
						}
					if (car_on == true){			//on a while time
						wait_time = wait_time_1;
						}
					control = true;					//exert control
					}
				alt_current = check_ad(alt_ad);		//check to see if car turned on/off
				if((alt_current > 10 && car_on == false) || (alt_current < 10 && car_on == true)){
					if(alt_current > 10){
						car_on = true;		//car on
						wait_time = wait_time_2;	//change to "just switched" time
						}
					if(alt_current < 10){
						car_on = false;
						soc = soc_check();	//car off
						wait_time = wait_time_2;	//"just switched" time
						}
					control = true;
				}
			}
			if(user_flag == true){		//if user interrupt was given
				user_flag = false;		//reset flag
				control = true;			//check control after leaving
				int_wdt_init();			//redundant, from previous iteration of prog.
				LCD_clear();
				LCD_main_menu(read_all, address_high_start, address_low_start, address_high_inc, address_low_inc);	//User interface functions will go here.
				if(term_read == 1){	//if terminal was accessed
					address_low_start = address_low_inc;	//adjust pointers
					address_high_start = address_high_inc;	
					term_read = 0;	//reset flag
					read_all = false;	//reset read-all flag
					}
				}
			if(control == true){
				fault = control_routine(over, car_on, wait_time); //change control
				address_low_inc = i2c_time(true, address_low_inc, address_high_inc);	//write to memory
				address_low_inc = i2c_data(car_on, over, fault, address_low_inc, address_high_inc);
				if ((address_low_inc + size) > 255){	//adjust pointer
					address_low_inc = 0x00;				//change high address
					address_high_inc = address_high_inc + 1;
					if (address_high_inc == 256){	//write around memory
						address_high_inc = 0;
					}
				}
				if((address_high_inc == address_high_start) && (address_low_inc == address_low_start)){
					read_all = true;	//tells terminal program to "read around" if memory is full.
				}
				control = false; //reset control flag
				time_track = 0;	//reset timeer.
				}
			LCD_clear(); //clear LCD screen
		}
	}



short check_timeoutflag(void){
	return timeoutflag;
	}
	
short write_timeoutflag(short tof){
	timeoutflag = tof;
	}

short check_twentyfour(void){
	return twentyfour;
	}

void write_twentyfour(short tf){
	twentyfour = tf;
	}
	
unsigned short check_override(void){
	return over;
	}

void write_override(unsigned short ovr){
	over = ovr;
}

unsigned short check_termread(){
	return term_read;
}

void write_term_read(unsigned short tr){
	term_read = tr;
}
